// CoVFW.cpp : Implementation of CoVFW
#include "stdafx.h"

EXTERN_C const IID DIID__oVFWEvents= 
	{ 0x54526101, 0xf0ca, 0x11d1, { 0x96, 0x9f, 0x0, 0x20, 0x18, 0x63, 0x16, 0x32 } };


#include <vfw.h>
#include "oVFW.h"
#include "CoVFW.h"


/////////////////////////////////////////////////////////////////////////////
// CoVFW


CoVFW::CoVFW()
{
	m_bWindowOnly = TRUE; 
	
	m_fOverlay= true;
	m_iVideo= 0;
	m_previewRate= 15;
	hWndC= NULL;
	pDriverInfo= new CComObject<CoVFWDriverInfo>;
	dynamic_cast<IoVFWDriverInfo*>(pDriverInfo)->AddRef();
}

CoVFW::~CoVFW()
{
	StopLiveVideo();

	// invalidate the object.
	pDriverInfo->index= -1;
	pDriverInfo->hWndC= NULL;
	dynamic_cast<IoVFWDriverInfo*>(pDriverInfo)->Release();
}

STDMETHODIMP CoVFW::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IoVFW,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CoVFW::OnDraw(ATL_DRAWINFO& di)
{
	return S_OK;
}

STDMETHODIMP CoVFW::get_iVideo(short * pVal)
{
	*pVal= m_iVideo;
	return S_OK;
}

STDMETHODIMP CoVFW::put_iVideo(short newVal)
{
	// check for validity.
	if (newVal<0 || newVal> 9)
		return Error( _T("oVFW: value out of range"), IID_IoVFW, CUSTOM_CTL_SCODE(9));

	m_iVideo= newVal;
	return S_OK;
}

STDMETHODIMP CoVFW::get_PreviewRate(short * pVal)
{
	*pVal= m_previewRate;
	return S_OK;
}

STDMETHODIMP CoVFW::put_PreviewRate(short newVal)
{
	if (newVal< 1 || newVal> 30)
		return Error( _T("oVFW: value out of range"), IID_IoVFW, CUSTOM_CTL_SCODE(9));

	m_previewRate= newVal;
	return S_OK;
}

STDMETHODIMP CoVFW::get_fOverlay(VARIANT_BOOL * pVal)
{
	*pVal= (VARIANT_BOOL)m_fOverlay;
	return S_OK;
}

STDMETHODIMP CoVFW::put_fOverlay(VARIANT_BOOL newVal)
{
	m_fOverlay= newVal?true:false;
	return S_OK;
}

STDMETHODIMP CoVFW::ShowLiveVideo(VARIANT_BOOL fShowLiveVideo, VARIANT_BOOL * fRC)
{
	if (fShowLiveVideo) 
		*fRC= (VARIANT_BOOL)StartLiveVideo();
	else 
		*fRC= (VARIANT_BOOL)StopLiveVideo();

	return S_OK;
}

bool CoVFW::StartLiveVideo()
{
	// already connected
	if (hWndC)
		return false;

	// connect to VFW.
	if (!AttachVFW())
		return false;


	// only try overlay if it exists!
	if (m_fOverlay) {
		VARIANT_BOOL fOverlay;
		pDriverInfo->get_SupportsOverlay(&fOverlay);
		m_fOverlay= fOverlay?true:false;
	}

	// scale the window.
	capPreviewScale(hWndC, TRUE);

	// resize window correctly again...
	::SetWindowPos(hWndC, NULL, 0, 0, w, h, SWP_NOZORDER | SWP_NOACTIVATE);

	if (m_fOverlay)  {
		if (!capOverlay(hWndC, TRUE))  // start the overlay.
			return false;
	}
	else {
		DWORD lRate= 1000L/ m_previewRate; // 1000/ (# of frames/sec)

		// I want to preview, if possible - overlay if needed
		capPreviewRate(hWndC, lRate); 
		if (!capPreview(hWndC, TRUE))
			return false;
	}

	return true;
}

bool CoVFW::StopLiveVideo()
{
	if (!hWndC)
		return false;

	if (!IsWindow(hWndC))
		return false;

	// turn both options off...
	capOverlay(hWndC, FALSE);
	capPreview(hWndC, FALSE);

	// detach from VFW.
	DetachVFW();	

	return true;
}

bool CoVFW::AttachVFW()
{	
	// get my own client rectangle.
	RECT rect;
	GetClientRect(&rect);

	w= short(rect.right- rect.left);
	h= short(rect.bottom- rect.top);

	// cast myself into a window handle.
	HWND hWndParent= HWND(*this);

	// attach window to VFW
	hWndC = capCreateCaptureWindow ( "oVFW", 
			WS_CHILD | WS_VISIBLE , 0, 0, w, h, (HWND)hWndParent, 0);

	// no free VFW windows.
	if (!hWndC)
		return false;

	// could not connect.
	if (!capDriverConnect(hWndC, m_iVideo))
		return false;

	pDriverInfo->index= m_iVideo;
	pDriverInfo->hWndC= hWndC;

	return true;
}

bool CoVFW::DetachVFW()
{
	capDriverDisconnect(hWndC); 
	::DestroyWindow(hWndC);
	hWndC= 0;

	return true;
}

STDMETHODIMP CoVFW::CaptureEditCopy(VARIANT_BOOL * fRC)
{
	*fRC= FALSE;
	if (hWndC) { // already attached.
		capGrabFrameNoStop(hWndC); // grab the frame.
		// copy to clipboard.
		*fRC= (VARIANT_BOOL)capEditCopy(hWndC);
	}
	else { //initiate a quick attach/capture/detach sequence.
		if (AttachVFW()) { // attach to VFW
			capGrabFrameNoStop(hWndC);	// grab the frame.
			// copy to clipboard.
			*fRC= (VARIANT_BOOL)capEditCopy(hWndC);	
			DetachVFW();			// detach from VFW.
		}
		else
			return S_OK; // failure.
	}

	// success after all...
	*fRC= TRUE;
	return S_OK;
}

STDMETHODIMP CoVFW::CaptureToBitmapFile(BSTR sFileName, VARIANT_BOOL * fRC)
{
	USES_CONVERSION;
	LPCTSTR szFileName= OLE2T(sFileName);

	*fRC= FALSE;
	if (hWndC) { // already attached.
		capGrabFrameNoStop(hWndC); // grab the frame.
		// copy to Bitmap file.
		*fRC= (VARIANT_BOOL)capFileSaveDIB(hWndC, szFileName);
	}
	else { //initiate a quick attach/capture/detach sequence.
		if (AttachVFW()) { // attach to VFW
			capGrabFrameNoStop(hWndC);	// grab the frame.

			// copy to Bitmap file.
			*fRC= (VARIANT_BOOL)capFileSaveDIB(hWndC, szFileName);   
			DetachVFW();			// detach from VFW.
		}
		else
			return S_OK; // failure.
	}

	// success after all...
	*fRC= TRUE;
	return S_OK;
}

STDMETHODIMP CoVFW::get_CurDriver(IoVFWDriverInfo * * pVal)
{
	// rougue DriverInfo object.
	if (!hWndC)
		return Error( _T("oVFW: Not attached to a VFW driver"), 
					  IID_IoVFW, CUSTOM_CTL_SCODE(1010));
	
	// get a pointer to the interface.
	IoVFWDriverInfo * pOut= dynamic_cast<IoVFWDriverInfo*>(pDriverInfo);
	pOut->AddRef();
	// the contents of where the parameter is pointing - will now point to the interface.
	*pVal= pOut;

	return S_OK;
}

STDMETHODIMP CoVFW::EnumDrivers()
{
	// typecast myself into a window handle.
	HWND hWndParent= HWND(*this);

	// create the enumeration window.
	HWND hWndEnum = capCreateCaptureWindow ( "VFW enumerating drivers", 
			WS_CHILD & ~WS_VISIBLE , 0, 0, 5, 5, (HWND)hWndParent, 1);

	// zero to 9.
	for (int iVid= 0; iVid< 10; iVid++) {
		BOOL rc= capDriverConnect(hWndEnum, iVid);
		// success
		if (rc) {
			CComObject<CoVFWDriverInfo> *pDriver= new CComObject<CoVFWDriverInfo>;

			CoVFWDriverInfo* newInfo= pDriver;
			newInfo->hWndC= hWndEnum;
			newInfo->index= iVid;

			pDriver->AddRef();
			IoVFWDriverInfo *pFireEvent= dynamic_cast<IoVFWDriverInfo *>(pDriver);
			Fire_DriversEnum(iVid, &pFireEvent);

			delete newInfo;
			
			// disconnect from the particular driver.
			capDriverDisconnect(hWndEnum); 
		}
	}

	// kill the VFW window.
	::DestroyWindow(hWndEnum);

	return S_OK;
}

BOOL CoVFW::IsWindow(HWND hWndC)
{
	return ::IsWindow(hWndC);
}
