package Sequence::Nucleotide::Subsequence;
# File: Sequence/Nucleotide/Subsequence.pm
use Sequence::Nucleotide;
use Carp;

use strict;
use vars '@ISA';
@ISA = 'Sequence::Nucleotide';

# constructor
sub new {
    my $class = shift;
    $class = ref($class) if ref($class);
    my ($parent,$start,$end) = @_;
    bless { parent => $parent,
	    start  => $start || 1,
	    end    => $end || $parent->length },$class;
}

# return the underlying sequence data
sub seq {
    my $self = shift;
    return '' unless $self->parent;
    my $data = $self->parent->seq;
    if ($self->end >= $self->start) {
	$data = substr($data,$self->start-1,$self->end-$self->start+1);
    } else {
	$data = substr($data,length($data)-$self->start,
		       1+length($data)-$self->end);
	$data = reverse($data);
	$data=~tr/GATCgatc/CTAGctag/;
    }
    return $data;
}

# return the type (DNA or RNA)
sub type {
    return shift()->parent->type(@_);
}

# return the parent object
sub parent {
    return shift()->{parent};
}

# the start of the subsequence in the parent
sub start {
    my $self = shift;
    return defined($_[0]) ? $self->{'start'} = $_[0] : $self->{'start'};
}

# the end of the subsequence in the parent
sub end {
    my $self = shift;
    return defined($_[0]) ? $self->{'end'} = $_[0] : $self->{'end'};
}

1;
